import os
import random
from datetime import time
from pathlib import Path

from django.http import HttpResponse

from ..utils.unionpay_utils import *

from main.settings import BASE_DIR
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
# 密钥地址
cert_path = main_config_dict.UNIONPAY_CERT_PATH or module_config_dict.UNIONPAY_CERT_PATH or (
        str(BASE_DIR) + "/config/acp_test_sign.pfx")

cert_password = main_config_dict.unionpay_cert_password

unionpay_version = main_config_dict.unionpay_version

# 商户ID(配置项)
unionpay_mer_id = main_config_dict.unionpay_mer_id

# 前台回调地址(支付成功回调成功)(配置项)（正式生产环境需修改）
unionpay_front_url = main_config_dict.unionpay_front_url

# 后台回调地址(配置项)（正式生产环境需修改）
unionpay_back_url = main_config_dict.unionpay_back_url

# 证书地址(配置项)（正式生产环境需修改）
unionpay_cert_path = main_config_dict.unionpay_cert_path

# 证书解密密码(根据实际去调配)(配置项) （正式生产环境需修改）
unionpay_cert_password = main_config_dict.unionpay_cert_password

# 是否开启测试模式(默认False)(配置项)（正式生产环境为False）
unionpay_debug = True

class PaymentUnionPayService:

    @staticmethod
    def get_uni_object():
        uni_pay = UnionPay(
            version=unionpay_version,
            mer_id=unionpay_front_url,
            front_url=unionpay_front_url,
            back_url=unionpay_back_url,
            backend_url=unionpay_cert_path,
            cert_path=cert_path,
            debug=unionpay_debug
        )
        return uni_pay

    # @staticmethod
    # 生成订单号（自定义）
    # def order_num(package_num, uid):
    #     '''
    #     商品代码后两位+下单时间后十二位+用户id后四位+随机数四位
    #     :param package_num: 商品代码
    #     :return: 唯一的订单号
    #     '''
    #     local_time = time.strftime('%Y%m%d%H%M%S', time.localtime(time.time()))[2:]
    #     result = str(package_num)[-2:] + local_time + uid[-4:] + str(random.randint(1000, 9999))
    #     return result


    # 账户充值（银联）  # uid参数可不传入，此处用于生成订单号
    @staticmethod
    def unipay(money, uid):
        # money = float(request.POST.get('money'))
        unipay = PaymentUnionPayService.get_uni_object()
        query_params = unipay.build_request_data(
            order_id="123456",  # 用户购买订单（每次不一样）
            txn_amt=int(float(money) * 100)  # 交易金额 单位分
        )
        # pay_html = unipay.pay_html(query_params)
        # rsp = HttpResponse()
        # rsp.content = pay_html
        return query_params


    # 充值成功后前台回调(银联)
    @staticmethod
    def uni_back(request, uid):
        if request.method == "POST":
            params = request.POST.dict()
            unipay = PaymentUnionPayService.get_uni_object(uid)
            res = unipay.verify_sign(params)
            if res:
                if unipay.verify_query(params['orderId'], params['txnTime']):  # 再次查询状态
                    return HttpResponse('充值成功')
            else:
                return HttpResponse('充值失败')


    # 充值成功后后台回调（银联）
    @staticmethod
    def uni_notify(request, uid):
        if request.method == "POST":
            params = request.POST.dict()
            unipay = PaymentUnionPayService.get_uni_object(uid)
            res = unipay.verify_sign(params)
            if res:
                status = unipay.verify_query(params['orderId'], params['txnTime'])  # 再次查询状态
                if status:
                    try:
                        # updata_data(uid, float(int(params['txnAmt']) / 100))
                        return HttpResponse('ok')
                    except Exception as e:
                        raise e
            else:
                return None, None
        else:
            params = request.GET.dict()
            for k, v in params.items():
                print(k, v, '\n')
        return HttpResponse('failed')



import logging
from pathlib import Path

from django.db import transaction
from django.forms import model_to_dict
from lxml import etree as et
import requests

from main.settings import BASE_DIR
from xj_enroll.service.enroll_services import EnrollServices
from xj_finance.services.finance_service import FinanceService
from xj_finance.services.finance_transact_service import FinanceTransactService
from xj_payment.models import PaymentPayment
from xj_thread.services.thread_item_service import ThreadItemService
from xj_user.services.user_service import UserService
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.wechat_utils import my_ali_pay

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

sub_appid = main_config_dict.wechat_merchant_app_id or module_config_dict.wechat_merchant_app_id or ""

sub_app_secret = main_config_dict.wechat_merchant_app_secret or module_config_dict.wechat_merchant_app_secret or ""

sub_mch_id = main_config_dict.wechat_merchant_mch_id or module_config_dict.wechat_merchant_mch_id or ""

trade_type = main_config_dict.wechat_trade_type or module_config_dict.wechat_trade_type or ""
# 交易类型，小程序取值：JSAPI

# 商品描述，商品简单描述
description = main_config_dict.wechat_body or module_config_dict.wechat_body or ""
# 标价金额，订单总金额，单位为分
total_fee = main_config_dict.wechat_total_fee or module_config_dict.wechat_total_fee or ""
# 通知地址，异步接收微信支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。
notify_url = main_config_dict.wechat_notify_url or module_config_dict.wechat_notify_url or ""

# 用户标识，trade_type=JSAPI，此参数必传，用户在商户appid下的唯一标识。
# print("<trade_type>", trade_type)

url = "https://api.mch.weixin.qq.com/v3/pay/partner/transactions/jsapi"

logger = logging.getLogger(__name__)


class Networkerror(Exception):
    def __init__(self, arg):
        self.args = arg


class PaymentWechatService:

    @staticmethod
    def get_user_info(code):
        # https://open.weixin.qq.com/connect/oauth2/authorize?appid=wx0c2a8db23b2e7c28&redirect_uri=REDIRECT_URI&response_type=code&scope=snsapi_userinfo&state=STATE#wechat_redirect
        req_params = {
            'appid': sub_appid,
            'secret': sub_app_secret,
            'js_code': code,
            'grant_type': 'authorization_code',
        }
        user_info = requests.get('https://api.weixin.qq.com/sns/jscode2session', params=req_params, timeout=3,
                                 verify=False)
        return user_info.json()

    # 微信小程序支付
    @staticmethod
    def payment_applets_pay(params):

        # out_trade_no = timezone.now().strftime('%Y%m%d%H%M%S') + ''.join(map(str, random.sample(range(0, 9), 4)))
        # data, err_txt = EnrollServices.enroll_detail(params['enroll_id'])
        # if err_txt:
        #     return "报名记录不存在"
        # sso_ret, err = UserSsoServeService.user_sso_to_user(data['user_id'])
        # if err:
        #     return "用户信息不存在"
        # print(sso_ret)
        # total_fee = float(params['total_fee']) * 100
        try:
            pay = my_ali_pay()
            order = pay.order.create(
                trade_type="JSAPI",  # 交易类型，小程序取值：JSAPI
                body=description,  # 商品描述，商品简单描述
                total_fee=int(params['total_fee']),  # 标价金额，订单总金额，单位为分
                notify_url=notify_url,  # 通知地址，异步接收微信支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。
                sub_mch_id=sub_mch_id,
                sub_appid=sub_appid,
                sub_user_id=params['openid'],  # 用户标识，trade_type=JSAPI，此参数必传，用户在商户appid下的唯一标识。
                # out_trade_no=out_trade_no
                out_trade_no=params["out_trade_no"]
            )
        except Exception as e:
            return {"error": str(e)}
        wxpay_params = pay.jsapi.get_jsapi_params(order['prepay_id'])
        # PaymentPayment.objects.create(**payment_data)
        # wxpay_params = PaymentWechatService.get_jsapi_params(order['prepay_id'])
        return wxpay_params

    # 余额支付
    @staticmethod
    def payment_balance_pay(params):
        balance = FinanceService.check_balance(account_id=params['user_id'], platform=params['platform'],
                                               platform_id=None,
                                               currency='CNY',
                                               sand_box=None)
        if balance['balance'] < int(params['total_amount']):
            return "余额不足"
        params['total_fee'] = float("-" + params['total_amount'])
        params['pay_mode'] = 'balance'
        return PaymentWechatService.payment_logic_processing(params)

    # @staticmethod
    # def get_jsapi_params(prepay_id, timestamp=None, nonce_str=None, jssdk=False):
    #     """
    #     获取 JSAPI 参数
    #
    #     :param prepay_id: 统一下单接口返回的 prepay_id 参数值
    #     :param timestamp: 可选，时间戳，默认为当前时间戳
    #     :param nonce_str: 可选，随机字符串，默认自动生成
    #     :param jssdk: 前端调用方式，默认使用 WeixinJSBridge
    #                   使用 jssdk 调起支付的话，timestamp 的 s 为小写
    #                   使用 WeixinJSBridge 调起支付的话，timeStamp 的 S 为大写
    #     :return: 参数
    #     """
    #     data = {
    #         'appId': sub_appid,
    #         'timeStamp': timestamp or to_text(int(time.time())),
    #         'nonceStr': nonce_str or random_string(32),
    #         'package': 'prepay_id={0}'.format(prepay_id),
    #     }
    #     # sign = calculate_sign(data, "POST", url, data['timeStamp'], data['nonceStr'])
    #     sign = get_pay_sign_info(data, prepay_id)
    #     logger.debug('JSAPI payment parameters: data = %s, sign = %s', data, sign)
    #     data['paySign'] = sign
    #     if jssdk:
    #         data['timestamp'] = data.pop('timeStamp')
    #     return data

    # 微信扫码支付
    @staticmethod
    def payment_scan_pay(params):
        pay = my_ali_pay()
        order = pay.order.create(
            trade_type="NATIVE",  # 交易类型，小程序取值：JSAPI
            body=description,  # 商品描述，商品简单描述
            total_fee=params['total_fee'],  # 标价金额，订单总金额，单位为分
            notify_url=notify_url,  # 通知地址，异步接收微信支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。
            sub_mch_id=sub_mch_id,
            sub_appid=sub_appid,
            out_trade_no=params['out_trade_no']
        )
        wxpay_params = pay.jsapi.get_jsapi_params(order['prepay_id'])
        # sign = calculate_sign(body, "POST", self.url, timestamp, nonce_str)
        return wxpay_params

    # 微信退款
    @staticmethod
    def payment_refund(params):
        # headers = {
        #     'Content-Type': 'application/json'
        # }
        # req_params = {
        #     'sub_mchid': sub_appid,
        #     'transaction_id': "4200001618202210284687786660",
        #     'out_refund_no': timezone.now().strftime('%Y%m%d%H%M%S') + ''.join(map(str, random.sample(range(0, 9), 4))),
        #     "amount": {
        #         "refund": 1,  # 退款金额
        #         "total": 1,
        #         "currency": "CNY"
        #     }
        # }
        # user_info = requests.post('https://api.mch.weixin.qq.com/v3/refund/domestic/refunds', params=req_params,
        #                           json=json.dumps(req_params))
        # print(user_info.json())
        pay = my_ali_pay()
        order = pay.refund.apply(
            total_fee=params['total_fee'],  # 标价金额，订单总金额，单位为分
            notify_url=notify_url,  # 通知地址，异步接收微信支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。
            sub_mch_id=sub_mch_id,
            sub_appid=sub_appid,
            out_trade_no=params['out_trade_no']
        )
        return None

    # 小程序回调
    @staticmethod
    def callback(_xml):
        """
        <xml><appid><![CDATA[wx56232dd67c7e5a18]]></appid> 微信分配的小程序ID
        <bank_type><![CDATA[CFT]]></bank_type>付款银行
        <cash_fee><![CDATA[1]]></cash_fee>现金支付金额订单现金支付金额
        <fee_type><![CDATA[CNY]]></fee_type>货币类型
        <is_subscribe><![CDATA[N]]></is_subscribe>用户是否关注公众账号，Y-关注，N-未关注
        <mch_id><![CDATA[1521497251]]></mch_id>微信支付分配的商户号
        <nonce_str><![CDATA[1546088296922]]></nonce_str>随机字符串，不长于32位
        <openid><![CDATA[oEHJT1opJZLYBWssRlyjq9bSdnao]]></openid>用户在商户appid下的唯一标识
        <out_trade_no><![CDATA[10657298351779092719122609746693]]></out_trade_no>商户系统内部订单号，要求32个字符内
        <result_code><![CDATA[SUCCESS]]></result_code>业务结果 SUCCESS/FAIL
        <return_code><![CDATA[SUCCESS]]></return_code>返回状态码 return_code
        <sign><![CDATA[2EB71F6237E04C3DA4B1509A502E8F62]]></sign>签名
        <time_end><![CDATA[20181229205830]]></time_end>支付完成时间
        <total_fee>1</total_fee>订单总金额，单位为分
        <trade_type><![CDATA[MWEB]]></trade_type>交易类型 JSAPI、NATIVE、APP
        <transaction_id><![CDATA[4200000224201812291041578058]]></transaction_id>微信支付订单号
        </xml>
        """

        # _xml = request.body
        # 拿到微信发送的xml请求 即微信支付后的回调内容
        xml = str(_xml, encoding="utf-8")
        return_dict = {}
        tree = et.fromstring(xml)
        # xml 解析
        return_code = tree.find("return_code").text
        try:
            if return_code == 'FAIL':
                # 官方发出错误
                return_dict['return_code'] = "SUCCESS"
                return_dict['return_msg'] = "OK"
                logging.error("微信支付失败")
                # return Response(return_dict, status=status.HTTP_400_BAD_REQUEST)
            elif return_code == 'SUCCESS':
                # 拿到自己这次支付的 out_trade_no
                out_trade_no = tree.find("out_trade_no").text  # 订单号
                total_fee = tree.find("total_fee").text  # 金额（单位分）
                transaction_id = tree.find("transaction_id").text  # 微信支付订单号
                appid = tree.find("appid").text  #
                # 报名回调
                data, err = EnrollServices.pay_call_back(out_trade_no)
                print("pay_call_back:", data, err)

                param = {
                    "out_trade_no": out_trade_no,
                    "total_fee": total_fee,
                    "transaction_id": transaction_id,
                    "appid": appid
                }
                PaymentWechatService.payment_logic_processing(param)
                return_dict['return_code'] = "SUCCESS"
                return_dict['return_msg'] = "OK"
        except Exception as e:
            return_dict['message'] = str(e)
        finally:
            xml_data = "<xml><return_code><![CDATA[{return_code}]]></return_code><return_msg><![CDATA[{return_msg}]]></return_msg> </xml> "
            kw = {'return_code': 'SUCCESS', 'return_msg': 'OK'}
            # 格式化字符串
            xml = xml_data.format(**kw)
            return xml

    # 支付逻辑处理
    @staticmethod
    def payment_logic_processing(param):
        sid = transaction.savepoint()
        try:
            project_name = ""
            summary = ""
            out_trade_no = param['out_trade_no']  # 订单号
            total_fee = param['total_fee']  # 金额（单位分）
            appid = param['appid']
            total_amount = int(total_fee) / 100  # 分转元
            transaction_id = 0
            if 'transaction_id' in param:
                transaction_id = param['transaction_id']  # 微信支付订单号
            finance_data = {
                "order_no": out_trade_no,
                "transact_id": transaction_id,
                "their_account_name": sub_appid,
                "platform": "muzpay",
                "amount": float("-" + str(total_amount)),
                "currency": "CNY",
                "pay_mode": "WECHAT",
            }
            # 根据订单号查询支付记录是否存在
            payment = PaymentPayment.objects.filter(order_no=int(out_trade_no)).first()
            if not payment:
                logging.info("payment_callback" + "支付记录不存在")
            payment_message = model_to_dict(payment)
            finance_data['account_id'] = payment_message['user_id']
            finance_data['enroll_id'] = payment_message['enroll_id']
            # 根据支付记录用户 查询用户基本信息
            user_set, err = UserService.user_basic_message(payment_message['user_id'])
            user_platform_set, platform_err = UserService.user_basic_message(nickname=sub_appid)
            if user_set:
                if payment_message['enroll_id']:
                    # 如果存在报名id 查询报名记录
                    enroll_set, err = EnrollServices.enroll_detail(payment_message['enroll_id'])
                    if enroll_set:
                        # paid_amount = enroll_set.get("paid_amount", 0) if enroll_set.get("paid_amount", 0) else 0
                        # 报名表支付状态修改
                        # enroll_data = {
                        #     "enroll_status_code": "43",
                        #     "paid_amount": float(paid_amount) + float(total_amount),
                        #     "unpaid_amount": 0
                        # }
                        # enroll, enroll_err_txt = EnrollServices.enroll_edit(enroll_data,
                        #                                                     payment_message['enroll_id'])
                        # if enroll_err_txt:
                        #     logging.info("payment_callback_enroll" + enroll_err_txt)
                        # 根据报名记录获取 信息模块项目基本信息
                        thread_set, err = ThreadItemService.detail(enroll_set['thread_id'])
                        if thread_set:
                            project_name = thread_set['title']
                summary = "【" + user_set['full_name'] + "】支付 【" + user_platform_set[
                    'full_name'] + "】项目名称【" + project_name + "】款项"
            finance_data['summary'] = summary
            # # TODO 拿到订单号后的操作 看自己的业务需求
            # 写入资金模块 （用户支付 (购买支出) ）
            funance_add_data, err_txt = FinanceTransactService.post(finance_data)
            if err_txt:
                logging.info("payment_callback" + err_txt)
            # 根据唯一交易id 查询主键id
            finance_data, err = FinanceTransactService.finance_transact_detailed(transaction_id)
            if finance_data:
                finance_data = model_to_dict(finance_data)
                payment_data = {
                    "transact_no": transaction_id,
                    "transact_id": finance_data['id'],
                    "order_status_id": "24"
                }
                # （平台的收入）
                platform_revenue_data = {
                    "account_id": finance_data['their_account'],
                    "their_account_id": finance_data['account'],
                    "enroll_id": finance_data['enroll_id'],
                    "platform": "muzpay",
                    "amount": float(total_amount),
                    "summary": "【" + user_set['full_name'] + "】支付 【" + user_platform_set[
                        'full_name'] + "】项目名称【" + project_name + "】款项",
                    "currency": "CNY",
                    "pay_mode": "BALANCE",
                }
                finance_platform_data_set, finance_platform_err = FinanceTransactService.post(platform_revenue_data)
                if finance_platform_err:
                    logging.info("payment_callback" + finance_platform_err)
                    raise Networkerror("Bad hostname")
                # 更改支付记录
                PaymentPayment.objects.filter(order_no=int(out_trade_no)).update(**payment_data)
        except Networkerror as t:
            # print(t)
            transaction.savepoint_rollback(sid)
        except Exception as e:
            # print(e)
            transaction.savepoint_rollback(sid)
            logging.info("payment_logic_processing" + e)



from django.http import JsonResponse, HttpResponse
from rest_framework import request
from rest_framework.views import APIView
from ..services.payment_wechat_service import PaymentWechatService
from ..utils.model_handle import parse_data


class PaymentWechat(APIView):
    # 获取唯一标识
    def get_user_info(self):
        code = self.GET.get('code', 0)
        wxpay_params = PaymentWechatService.get_user_info(code)

        return JsonResponse({
            'err': 0,
            'msg': 'OK',
            'data': wxpay_params
        })

    # 小程序支付
    def payment_applets_pay(self):
        params = parse_data(self)
        wxpay_params = PaymentWechatService.payment_applets_pay(params)
        return JsonResponse({
            'err': 0,
            'msg': 'OK',
            'data': wxpay_params
        })

    # 扫码支付
    def payment_scan_pay(self):
        params = self.POST
        wxpay_params = PaymentWechatService.payment_scan_pay(params)
        return JsonResponse({
            'err': 0,
            'msg': 'OK',
            'data': wxpay_params
        })

    # 回调接口
    def callback(self):
        _xml = self.body
        wxpay_params = PaymentWechatService.callback(_xml)
        return HttpResponse(wxpay_params)
        # return JsonResponse(wxpay_params)
        # return JsonResponse({
        #     'err': 0,
        #     'msg': 'OK',
        #     'data': wxpay_params
        # })



from django.db import models


# Create your models here.

# 1、Payment_Status  支付状态表 [NF1]
class PaymentStatus(models.Model):
    payment_status = models.CharField(verbose_name='支付状态', max_length=128, blank=True, null=True, db_index=True)
    group = models.CharField(verbose_name='分组', max_length=128, blank=True, null=True, db_index=True)
    description = models.CharField(verbose_name='描述', max_length=128, blank=True, null=True, db_index=True)

    class Meta:
        db_table = 'payment_status'
        verbose_name_plural = "01. 支付状态表"

    def __str__(self):
        # return f"{self.user_name}({self.full_name})"
        return f"{self.payment_status}"


# 2、payment_payment  支付订单记录表 [NF1]
class PaymentPayment(models.Model):
    # id = models.AutoField(verbose_name='id', blank=True, null=False, db_index=True)
    order_no = models.BigIntegerField(verbose_name='订单号', blank=True, null=False, db_index=True)
    transact_no = models.IntegerField(verbose_name='流水号', blank=True, null=False, db_index=True)
    transact_id = models.IntegerField(verbose_name='资金id', blank=True, null=True, db_index=True)
    enroll_id = models.IntegerField(verbose_name='报名id', blank=True, null=False, db_index=True)
    order_id = models.IntegerField(verbose_name='订单id', blank=True, null=False, db_index=True)
    user_id = models.IntegerField(verbose_name='用户id', blank=True, null=True, db_index=True)
    subject = models.CharField(verbose_name='商品标题', max_length=128, blank=True, null=True, db_index=True)
    total_amount = models.DecimalField(verbose_name='订单金额', max_digits=11, decimal_places=2, blank=True, null=True)
    buyer_pay_amount = models.DecimalField(verbose_name='用户支付金额', max_digits=11, decimal_places=2, blank=True,
                                           null=True)
    point_amount = models.DecimalField(verbose_name='积分点(集分宝)抵扣金额', max_digits=11, decimal_places=2, blank=True,
                                       null=True)
    invoice_amount = models.DecimalField(verbose_name='开票金额', max_digits=11, decimal_places=2, blank=True, null=True)
    price_off_amount = models.DecimalField(verbose_name='折扣金额', max_digits=11, decimal_places=2, blank=True, null=True)
    payment_type = (
        ("UNKNOWN", 'UNKNOWN'),
        ("ALIPAY", 'ALIPAY'),
        ("WECHAT", 'WECHAT'),
        ("UNION_PA", 'UNION_PA'),
    )
    pay_mode = models.CharField(verbose_name='支付类型', max_length=128, choices=payment_type)
    order_status_id = models.IntegerField(verbose_name='订单状态（十数法）', blank=True, null=True, db_index=True)
    payment_status = models.ForeignKey(PaymentStatus, verbose_name='支付状态', blank=True, null=True,
                                       on_delete=models.DO_NOTHING)
    nonce_str = models.JSONField(verbose_name='临时字段', max_length=128, blank=True, null=True, db_index=True)
    order_time = models.DateTimeField(verbose_name='订单创建时间', blank=True, null=True)
    create_time = models.DateTimeField(verbose_name='创建时间', auto_now_add=True, )
    modify_time = models.DateTimeField(verbose_name='修改时间', auto_now=True, blank=True, null=True, )
    payment_time = models.DateTimeField(verbose_name='支付时间', blank=True, null=True)
    refunt_time = models.DateTimeField(verbose_name='退款时间', blank=True, null=True)
    close_time = models.DateTimeField(verbose_name='退款时间', blank=True, null=True)
    voucher_detail = models.JSONField(verbose_name='优惠券详细信息', max_length=128, blank=True, null=True, db_index=True)
    snapshot = models.CharField(verbose_name='商品快照', max_length=128, blank=True, null=True, db_index=True)
    more = models.CharField(verbose_name='辅助信息', max_length=128, blank=True, null=True, db_index=True)

    class Meta:
        db_table = 'payment_payment'
        verbose_name_plural = "02. 支付订单记录表"

    def __str__(self):
        # return self.subject if self.subject else ""
        return f"{self.order_no}"



#!/usr/bin/python3
import os
import requests
import json
import logging
# from __future__ import absolute_import
from datetime import datetime, timedelta
from pathlib import Path

from django.http import JsonResponse

from main.settings import BASE_DIR
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.base import WechatPayDALBase

pay_logger = logging.getLogger('pay')

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

sub_appid = main_config_dict.wechat_merchant_app_id or module_config_dict.wechat_merchant_app_id or ""

sub_app_secret = main_config_dict.wechat_merchant_app_secret or module_config_dict.wechat_merchant_app_secret or ""

sub_mch_id = main_config_dict.wechat_merchant_mch_id or module_config_dict.wechat_merchant_mch_id or ""

apiv3_secret = main_config_dict.wechat_apiv3_secret or module_config_dict.wechat_apiv3_secret or ""

trade_type = main_config_dict.wechat_trade_type or module_config_dict.wechat_trade_type or ""
# 交易类型，小程序取值：JSAPI

# 商品描述，商品简单描述
description = main_config_dict.wechat_body or module_config_dict.wechat_body or ""
# 标价金额，订单总金额，单位为分
total_fee = main_config_dict.wechat_total_fee or module_config_dict.wechat_total_fee or ""
# 通知地址，异步接收微信支付结果通知的回调地址，通知url必须为外网可访问的url，不能携带参数。
notify_url = main_config_dict.wechat_notify_url or module_config_dict.wechat_notify_url or ""

private_key_path = main_config_dict.wechat_merchant_private_key_file or module_config_dict.wechat_merchant_private_key_file or (
        str(BASE_DIR) + "/config/apiclient_key.pem")
# 读取文件获取密钥
private_key = open(private_key_path, encoding="utf-8").read() if os.path.exists(private_key_path) else ""

# 商户证书序列号
serial_no = '548FB2EFC2CBBA142124DB82AED2B539977B062E.'

class WechatPayDAL(WechatPayDALBase):
    def get_official_cert(self):
        '''
        获取微信更新证书并保存
        '''
        url = 'https://api.mch.weixin.qq.com/v3/certificates'
        headers = self.make_headers_v3(url)
        rsp = requests.get(url, headers=headers)
        pay_logger.info('rsp:{}|{}'.format(rsp.status_code, rsp.content))
        rdct = rsp.json()
        for info in rdct['data']:
            ret = self.decrypt_v3(info)
            fpath = 'wechat_official_cert_{}.pem'.format(info['serial_no'])
            with open(fpath, 'wb') as ofile:
                ofile.write(ret)

        return fpath

    def create_order_info(self):
        '''
        创建微信预支付订单, 注意包含两次签名过程:
        首次签名用于请求微信后端获取prepay_id
        二次签名信息返回客户端用于调起SDK支付
        '''
        wechat = WechatPayDALBase(
            mch_appid=sub_mch_id,
            mchid=sub_appid,
            v3key=apiv3_secret,
            serial_no=serial_no,
            client_key=private_key_path
        )
        url = 'https://api.mch.weixin.qq.com/v3/pay/partner/transactions/jsapi'
        ndt = datetime.now()
        out_trade_no = self.generate_partner_trade_no(ndt)
        data = {
            'mchid': sub_mch_id,
            'out_trade_no': out_trade_no,
            'appid': sub_appid,
            'description': description,
            'notify_url': notify_url,
            'amount': {
                'currency': 'CNY',
                'total': int(total_fee),
            },
            'time_expire': (ndt + timedelta(minutes=5)).strftime('%Y-%m-%dT%H:%M:%S+08:00')
        }
        jdata = json.dumps(data, separators=[',', ':'])
        headers = {'Content-Type': 'application/json'}
        # 第一次签名, 直接请求微信后端
        headers = wechat.make_headers_v3(url, headers=headers, body=jdata, method='POST')
        rsp = requests.post(url, headers=headers, data=jdata)
        pay_logger.info('rsp:{}|{}'.format(rsp.status_code, rsp.text))
        rdct = rsp.json()
        print(rdct)
        # 第二次签名, 返回给客户端调用
        sign_info = wechat.get_pay_sign_info(rdct['prepay_id'])

        return JsonResponse(sign_info)
        # return sign_info

    def query_order(self, out_trade_no):
        '''
        查询指定订单信息
        '''
        url = f'https://api.mch.weixin.qq.com/v3/pay/transactions/out-trade-no/{out_trade_no}?mchid={self.mchid}'
        headers = self.make_headers_v3(url)
        rsp = requests.get(url, headers=headers)
        pay_logger.info('out_trade_no:{}, rsp:{}|{}'.format(out_trade_no, rsp.status_code, rsp.text))
        rdct = rsp.json()
        return rdct



import json
import os
import time

import rsa
import base64
import logging
from pathlib import Path

from Cryptodome.Hash import SHA256
from Cryptodome.PublicKey import RSA
from Cryptodome.Signature import pkcs1_15
from celery.utils.serialization import b64encode
from cryptography.hazmat.primitives.asymmetric.padding import PKCS1v15
from wechatpy import WeChatPay
from wechatpy.utils import to_binary, to_text
from main.settings import BASE_DIR
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
# 服务商商户的APPID
app_id = main_config_dict.wechat_service_app_id or module_config_dict.wechat_service_app_id or ""

app_secret = main_config_dict.wechat_service_app_secret or module_config_dict.wechat_service_app_secret or ""

mch_id = main_config_dict.wechat_service_mch_id or module_config_dict.wechat_service_mch_id or ""

merchant_key = main_config_dict.wechat_service_merchant_key or module_config_dict.wechat_service_merchant_key or ""

sub_appid = main_config_dict.wechat_merchant_app_id or module_config_dict.wechat_merchant_app_id or ""

sub_app_secret = main_config_dict.wechat_merchant_app_secret or module_config_dict.wechat_merchant_app_secret or ""

sub_mch_id = main_config_dict.wechat_merchant_mch_id or module_config_dict.wechat_merchant_mch_id or ""

apiv3_secret = main_config_dict.wechat_apiv3_secret or module_config_dict.wechat_apiv3_secret or ""
# 密钥地址
private_key_path = main_config_dict.wechat_merchant_private_key_file or module_config_dict.wechat_merchant_private_key_file or (
        str(BASE_DIR) + "/config/apiclient_key.pem")
# 读取文件获取密钥
private_key = open(private_key_path, encoding="utf-8").read() if os.path.exists(private_key_path) else ""

url = "https://api.mch.weixin.qq.com/v3/pay/partner/transactions/jsapi"

logger = logging.getLogger(__name__)


def my_ali_pay():
    wechat = WeChatPay(
        appid=app_id,
        sub_appid=sub_appid,
        api_key=merchant_key,
        mch_id=mch_id,
        sub_mch_id=sub_mch_id,
        mch_key=apiv3_secret,
    )
    return wechat

# def format_url(params, api_key=None):
#     data = [to_binary('{0}={1}'.format(k, params[k])) for k in sorted(params) if params[k]]
#     if api_key:
#         data.append(to_binary('key={0}'.format(api_key)))
#     return b"&".join(data)


# def sign_str(method, url, timestamp, nonce_str, body):
#     """
#     拼接sign字符串
#     """
#     # sign_list = [
#     #     method,
#     #     url,
#     #     timestamp,
#     #     nonce_str,
#     #     body
#     # ]
#     sign_list = [
#         body['appId'],
#         body['timeStamp'],
#         body['nonceStr'],
#         body['package'],
#     ]
#     # print(sign_list)
#     return '\n'.join(sign_list) + '\n'


# 生成sign
# def calculate_sign(body, method, url, timestamp, nonce_str):
# a = sign_str(method, url, timestamp, nonce_str, json.dumps(body))
# signer = pkcs1_15.new(RSA.importKey(open(r"apiclient_key.pem").read()))
# a = sign_str(method, url, timestamp, nonce_str, json.dumps(body))
# a = sign_str(method, url, timestamp, nonce_str, body)
# signs = rsa_sign(private_key, a)
# cipher = SHA256.new(private_key)
# cipher_text = base64.encodebytes(cipher.encrypt(a))
# signer = pkcs1_15.new(RSA.importKey(private_key))
# signature = signer.sign(SHA256.new(a.encode("utf-8")))
# sign = base64.b64encode(signature).decode("utf-8").replace("\n", "")
# sign = cipher_text
# return None


# def rsa_sign(private_key, sign_str):
#     message = sign_str.encode('UTF-8')
#     signature = private_key.sign(data=message, padding=PKCS1v15(), algorithm=SHA256())
#     sign = b64encode(signature).decode('UTF-8').replace('\n', '')
#     return sign
#
#
# def get_pay_sign_info(data, prepay_id):
#     content = '{}\n{}\n{}\n{}\n'.format(data['appId'], data['timeStamp'], data['nonceStr'], data['package'])
#     signer = pkcs1_15.new(RSA.importKey(private_key))
#     print(private_key)
#     digest = SHA256.new(content.encode('utf-8'))
#     sign_v = base64.b64encode(signer.sign(digest)).decode('utf-8')
#     return {
#         'appid': data['appId'],
#         'timestamp': data['timeStamp'],
#         'noncestr': data['nonceStr'],
#         # 'prepay_id': prepay_id,
#         # 'package': 'Sign=WXPay',
#         "package": data['package'],
#         'sign': sign_v,
#     }



import logging
import random
from datetime import datetime

import pytz
import requests
import time

from django.forms import model_to_dict
from django.utils import timezone

from xj_finance.services.finance_service import FinanceService
from xj_finance.services.finance_transact_service import FinanceTransactService
from xj_enroll.service.enroll_services import EnrollServices
from xj_thread.services.thread_item_service import ThreadItemService
from xj_user.services.user_service import UserService
from xj_payment.models import PaymentPayment
from xj_user.services.user_sso_serve_service import UserSsoServeService
from ..services.payment_wechat_service import PaymentWechatService


class PaymentService:
    # 支付总接口
    @staticmethod
    def pay(params):
        # print(params)
        data = params
        # data['total_fee'] = float(params['total_amount']) * 100  # 元转分
        payment_data = None
        out_trade_no = timezone.now().strftime('%Y%m%d%H%M%S') + ''.join(
            map(str, random.sample(range(0, 9), 4)))  # 随机生成订单号
        params['out_trade_no'] = out_trade_no
        if params['enroll_id']:
            enroll_data, err_txt = EnrollServices.enroll_detail(params['enroll_id'])  # 判断是否是报名订单
            if err_txt:
                return "报名记录不存在"
            data['enroll_id'] = enroll_data['id']
            data['user_id'] = enroll_data['user_id']
            data['total_fee'] = float(enroll_data['unpaid_amount']) * 100  # 元转分
        # 单点登录信息
        sso_data, err = UserSsoServeService.user_sso_to_user(data['user_id'])
        if err:
            return "单点登录记录不存在"
        sso_data = model_to_dict(sso_data)
        data['openid'] = sso_data['sso_unicode']
        tz = pytz.timezone('Asia/Shanghai')
        # 返回时间格式的字符串
        # now_time = timezone.now().astimezone(tz=tz)
        # now_time_str = now_time.strftime("%Y.%m.%d %H:%M:%S")
        # 返回datetime格式的时间
        now_time = timezone.now().astimezone(tz=tz).strftime("%Y-%m-%d %H:%M:%S")
        now = datetime.strptime(now_time, '%Y-%m-%d %H:%M:%S')
        payment_data = {
            "order_no": out_trade_no,
            "enroll_id": data['enroll_id'],
            "user_id": data['user_id'],
            "total_amount": int(data['total_fee']),
            "create_time": now,
        }
        data['platform'] = 'muzpay'
        data['currency'] = 'CNY'

        if params.get("total_amount", 0):
            data['total_fee'] = float(params.get("total_amount", 0)) * 100  # 元转分

        PaymentPayment.objects.create(**payment_data)
        # 支付方式检查
        if params['payment_method'] == "applets":  # 微信小程序支付

            payment = PaymentWechatService.payment_applets_pay(data)

        elif params['payment_method'] == "balance":  # 余额支付
            payment = PaymentWechatService.payment_balance_pay(data)

        return payment

    # 退款总接口
    @staticmethod
    def refund(params):
        data = params
        data['total_fee'] = float(params['total_amount']) * 100  # 元转分
        data['refund_fee'] = float(params['refund_amount']) * 100  # 元转分
        data['out_trade_no'] = timezone.now().strftime('%Y%m%d%H%M%S') + ''.join(
            map(str, random.sample(range(0, 9), 4)))
        payment = PaymentWechatService.payment_refund(data)
        return payment



from xj_finance.services.finance_transact_service import FinanceTransactService

from ..models import PaymentOrder


class PaymentFinanceService:
    @staticmethod
    def post(out_trade_no):
        """
        their_account_id 对方账户ID
        platform 平台
        amount 金额
        currency 币种
        pay_mode 支付方式
        summary 摘要
        :return:
        """
        payment_message = PaymentOrder.objects.first(order_no=out_trade_no).first()
        data = {
            'platform_order_id': payment_message['order_no'],
            'total_amout':payment_message['total_amout'],
            # 'platform_id'
        }
        FinanceTransactService.create(data)

